package WebService::Rackspace::CloudFiles::DateTime;
use DateTime;
use parent 'DateTime';

1;

__END__

WebService::Rackspace::CloudFiles::DateTime -- A subclass of DateTime
 
=head1 DESCRIPTION
 
This class is empty except for being a subclass of DateTime.
